using System;
using System.IO;
using System.Text;
using System.Xml;
using System.Xml.XPath;

namespace HtmlScrubber
{
	public abstract class TransformRequestFilterBase : Stream
	{
		protected Stream originalStream;
		protected Stream filteredStream;

		protected abstract ContentTransformation[] GetTransformations();
		
		public TransformRequestFilterBase(Stream original)
		{
			this.originalStream = original;
		}

		private void ReplaceStreamWithFilteredStream(Stream original)
		{
			XmlDocument doc = new XmlDocument();
			doc.Load(original);
			
			XPathNavigator navigator = doc.CreateNavigator();

			if(!IsEditPostOrNewPostMethod(navigator))
			{
				this.filteredStream = new MemoryStream(Encoding.UTF8.GetBytes(doc.OuterXml));
				return;
			}

			XPathNavigator node = navigator.SelectSingleNode("//member/name[.='description']/../value/string");
			if (node == null)
			{
				node = navigator.SelectSingleNode("//member/name[.='description']/../value");
			}

			if(node == null)
			{
				this.filteredStream = original;
					return;
			}

			string contents = node.Value;

			ContentTransformation[] transforms = GetTransformations();
			foreach (ContentTransformation replacement in transforms)
				contents = replacement.Transform(contents);

			node.SetValue(contents);

			this.filteredStream = new MemoryStream(Encoding.UTF8.GetBytes(doc.OuterXml));
		}

		static bool IsEditPostOrNewPostMethod(XPathNavigator navigator)
		{
			XPathNavigator methodCallNode = navigator.SelectSingleNode("//methodCall/methodName");

			if (methodCallNode == null)
				return false;

			switch(methodCallNode.Value)
			{
				case "metaWeblog.newPost":
				case "metaWeblog.editPost":
					return true;
				default:
					return false;
			}
		}

		private Stream FilteredStream
		{
			get
			{
				if(this.filteredStream == null)
					ReplaceStreamWithFilteredStream(this.originalStream);
				return filteredStream;
			}
		}

		/// <summary>
		/// When overridden in a derived class, gets a value indicating whether the current stream supports reading.
		/// </summary>
		/// <value></value>
		/// <returns>true if the stream supports reading; otherwise, false.</returns>
		/// <filterPriority>1</filterPriority>
		public override bool CanRead
		{
			get { return true; }
		}

		/// <summary>
		/// When overridden in a derived class, gets a value indicating whether the current stream supports seeking.
		/// </summary>
		/// <value></value>
		/// <returns>true if the stream supports seeking; otherwise, false.</returns>
		/// <filterPriority>1</filterPriority>
		public override bool CanSeek
		{
			get { return false; }
		}

		/// <summary>
		/// When overridden in a derived class, gets a value indicating whether the current stream supports writing.
		/// </summary>
		/// <value></value>
		/// <returns>true if the stream supports writing; otherwise, false.</returns>
		/// <filterPriority>1</filterPriority>
		public override bool CanWrite
		{
			get { return false; }
		}

		/// <summary>
		/// When overridden in a derived class, gets the length in bytes of the stream.
		/// </summary>
		/// <value></value>
		/// <returns>A long value representing the length of the stream in bytes.</returns>
		/// <filterPriority>1</filterPriority>
		public override long Length
		{
			get
			{
				return this.FilteredStream.Length;
			}
		}

		/// <summary>
		/// When overridden in a derived class, gets or sets the position within the current stream.
		/// </summary>
		/// <value></value>
		/// <returns>The current position within the stream.</returns>
		/// <filterPriority>1</filterPriority>
		public override long Position
		{
			get { return FilteredStream.Position; }
			set { throw new InvalidOperationException("Cannot set the position of a filter stream"); }
		}

		/// <summary>
		/// When overridden in a derived class, reads a sequence of bytes from the current stream and 
		/// advances the position within the stream by the number of bytes read.
		/// </summary>
		/// <param name="buffer">An array of bytes. When this method returns, the buffer contains 
		/// the specified byte array with the values between <paramref name="offset"/> and 
		/// (<paramref name="offset"/> + <paramref name="count"/> - 1) replaced by the bytes read 
		/// from the current source.</param>
		/// <param name="offset">The zero-based byte offset in <paramref name="buffer"/> at which to 
		/// begin storing the data read from the current stream.</param>
		/// <param name="count">The maximum number of bytes to be read from the current stream.</param>
		/// <returns>
		/// The total number of bytes read into the buffer. This can be less than the number of bytes 
		/// requested if that many bytes are not currently available, or zero (0) if the end of the stream has been reached.
		/// </returns>
		/// <filterPriority>1</filterPriority>
		public override int Read(byte[] buffer, int offset, int count)
		{
			return this.FilteredStream.Read(buffer, offset, count);
		}

		public override long Seek(long offset, SeekOrigin direction)
		{
			throw new NotSupportedException();
		}

		public override void SetLength(long length)
		{
			throw new NotSupportedException();
		}

		public override void Close()
		{
			FilteredStream.Close();
		}

		public override void Flush()
		{
			FilteredStream.Flush();
		}

		public override void Write(byte[] buffer, int offset, int count)
		{
			throw new NotSupportedException();
		}
	}
}
