using System;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;
using System.Xml.XPath;

namespace HtmlScrubber
{
	/// <summary>
	/// Transforms cleaned up content back into what WLW expects.
	/// </summary>
	public class WLWResponseFilter : Stream
	{
		private StringBuilder responseText = new StringBuilder();
		private Stream responseStream;
		private bool foundMethodResponseEndTag = false;
		private static Regex editableContentRegexCleanup = new Regex(@"(?<pre><div class="".*?) id-(?<id>.*?) contenteditable-(?<contenteditable>[a-z]*?)", RegexOptions.Compiled | RegexOptions.IgnoreCase);

		public WLWResponseFilter(Stream original)
		{
			this.responseStream = original;
		}

		/// <summary>
		/// When overridden in a derived class, gets a value indicating whether the current stream supports reading.
		/// </summary>
		/// <value></value>
		/// <returns>true if the stream supports reading; otherwise, false.</returns>
		/// <filterPriority>1</filterPriority>
		public override bool CanRead
		{
			get { return false; }
		}

		/// <summary>
		/// When overridden in a derived class, gets a value indicating whether the current stream supports seeking.
		/// </summary>
		/// <value></value>
		/// <returns>true if the stream supports seeking; otherwise, false.</returns>
		/// <filterPriority>1</filterPriority>
		public override bool CanSeek
		{
			get { return false; }
		}

		/// <summary>
		/// When overridden in a derived class, gets a value indicating whether the current stream supports writing.
		/// </summary>
		/// <value></value>
		/// <returns>true if the stream supports writing; otherwise, false.</returns>
		/// <filterPriority>1</filterPriority>
		public override bool CanWrite
		{
			get { return true; }
		}

		/// <summary>
		/// When overridden in a derived class, gets the length in bytes of the stream.
		/// </summary>
		/// <value></value>
		/// <returns>A long value representing the length of the stream in bytes.</returns>
		/// <filterPriority>1</filterPriority>
		public override long Length
		{
			get
			{
				return this.responseStream.Length;
			}
		}

		/// <summary>
		/// When overridden in a derived class, gets or sets the position within the current stream.
		/// </summary>
		/// <value></value>
		/// <returns>The current position within the stream.</returns>
		/// <filterPriority>1</filterPriority>
		public override long Position
		{
			get { return responseStream.Position; }
			set { throw new InvalidOperationException("Cannot set the position of a filter stream"); }
		}

		public override int Read(byte[] buffer, int offset, int count)
		{
			throw new InvalidOperationException("Cannot Seek");
		}

		public override long Seek(long offset, SeekOrigin direction)
		{
			throw new InvalidOperationException("Cannot Seek");
		}

		public override void SetLength(long length)
		{
			this.responseStream.SetLength(length);
		}

		public override void Close()
		{
			responseStream.Close();
		}

		public override void Flush()
		{
			if(!foundMethodResponseEndTag)
			{
				FlushContents(this.responseText.ToString());
			}
		}

		private void FlushContents(string contents)
		{
			XmlDocument doc = new XmlDocument();
			doc.LoadXml(contents);
			XPathNavigator navigator = doc.CreateNavigator();
			XPathNavigator node = navigator.SelectSingleNode("//member/name[.='description']/../value/string");
			if (node == null)
			{
				node = navigator.SelectSingleNode("//member/name[.='description']/../value");
			}

			if (node != null)
			{
				string nodeContent = node.Value;

				//Do transforms and then write everything to the response stream.
				Match match = editableContentRegexCleanup.Match(nodeContent);
				if (match.Success)
					nodeContent =
						editableContentRegexCleanup.Replace(nodeContent, @"${pre}"" id=""${id}"" contenteditable=""${contenteditable}");

				node.SetValue(nodeContent);
			}

			byte[] data = Encoding.UTF8.GetBytes(doc.OuterXml);
			this.responseStream.Write(data, 0, data.Length);
			foundMethodResponseEndTag = true;
		}

		/// <summary>
		/// When overridden in a derived class, writes a sequence of bytes to the current stream and advances the current position within this stream by the number of bytes written.
		/// </summary>
		/// <param name="buffer">An array of bytes. This method copies <paramref name="count"/> bytes from <paramref name="buffer"/> to the current stream.</param>
		/// <param name="offset">The zero-based byte offset in <paramref name="buffer"/> at which to begin copying bytes to the current stream.</param>
		/// <param name="count">The number of bytes to be written to the current stream.</param>
		/// <filterPriority>1</filterPriority>
		public override void Write(byte[] buffer, int offset, int count)
		{
			string text = Encoding.UTF8.GetString(buffer, offset, count);
			this.responseText.Append(text);
			string fullResponseContents = responseText.ToString();
			
			//I need to check the full contents here because a write on a large 
			//post could potentially end in the middle of that tag.
			if (fullResponseContents.EndsWith("<methodResponse>"))
			{
				FlushContents(fullResponseContents);
			}
		}
	}
}
